/* eslint-disable react/prop-types */
/* eslint-disable react/destructuring-assignment */
/**
 *
 * NavContainer
 *
 */

import React, { memo, useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { createStructuredSelector } from 'reselect';
import { compose } from 'redux';

import injectSaga, { useInjectSaga } from 'utils/injectSaga';
import injectReducer, { useInjectReducer } from 'utils/injectReducer';
import { Container, Row, Col, Button } from 'react-bootstrap';
import { motion } from 'framer-motion';
import Tour from 'reactour';
import styled from 'styled-components';
import makeSelectNavContainer from './selectors';
import reducer from './reducer';
import saga from './saga';
import MenuToggleComponent from '../../components/shared/MenuToggleComponent/Loadable';
import ReusableTooltip from '../../components/shared/ReusableTooltip/Loadable';

import history from '../../utils/history';
import getSteps from '../../utils/tourSteps';
import { getUserWalkthroughAction, saveUserWalkthroughAction } from './actions';
import { urlSwitcher } from '../../utils/urlSwitcher';
import makeSelectReusableNotificationContainer from '../ReusableNotificationContainer/selectors';
import reusableNotificationContainerReducer from '../ReusableNotificationContainer/reducer';
import reusableNotificationContainerSaga from '../ReusableNotificationContainer/saga';
import ReusableNotificationContainer from '../ReusableNotificationContainer/index';
/**
 *
 * @param {function} onClick
 * @param {string} icon
 * @param {string} path
 * @param {string} id
 */
const NavButton = ({ onClick, icon, path, id, className }) => (
  <ReusableTooltip
    id={`${id}-button`}
    content={id.replace('-', ' ')}
    place="right"
  >
    <Button
      bsPrefix={`nav-button ${className || ''}`}
      style={
        history.location.pathname === path
          ? { borderRight: '5px solid #23a5da', color: '#23a5da' }
          : { borderRight: '5px solid #395f60', color: '#fff' }
      }
      onClick={onClick}
    >
      <i className={icon} />
    </Button>
  </ReusableTooltip>
);

const getTourKey = path => {
  switch (path) {
    case '/':
      return 'computeComparison';
    case '/storage-costings':
      return 'storageCostings';
    case '/pod-calculator':
      return 'storageCostings';
    case '/database-costings':
      return 'databaseCostings';
    default:
      break;
  }
};

const StyledNavCol = styled.div`
  display: flex;
  flex-direction: column;
  height: 100%;
  min-width: 60px;
  max-width: 60px;
  width: 60px;
  z-index: 5;
  background-color: ${props => props.sidenavBackgroundColor};
`;

/**
 *
 * @param {node} children
 * @param {boolean} openMenu
 * @param {function} handleToggle
 */
function NavContainer(props) {
  const [steps, setSteps] = useState([]);
  const [isTourOpen, setIsTourOpen] = useState(false);
  const [tourKey, setTourKey] = useState('');

  useEffect(() => {
    props.dispatch(getUserWalkthroughAction());
  }, []);

  useEffect(() => {
    setTourKey(getTourKey(history.location.pathname));
  }, [history?.location?.pathname, props?.navContainer?.hasVisited]);

  useEffect(() => {
    const pageSteps = getSteps(history?.location?.pathname);
    setSteps(pageSteps);
    if (
      tourKey !== '' &&
      Object.keys(props?.navContainer?.hasVisited).length !== 0
    ) {
      setIsTourOpen(
        props?.navContainer?.hasVisited[tourKey]
          ? !props?.navContainer?.hasVisited[tourKey]
          : true,
      );
    }
  }, [tourKey]);

  return (
    <>
      <ReusableNotificationContainer
        dispatch={props.dispatch}
        reusableNotificationContainer={props.reusableNotificationContainer}
        notifications={[]}
      />
      <Container
        fluid
        as={Row}
        noGutters
        className="m-0 p-0 w-100 body-content"
        style={{ backgroundColor: '#395f60' }}
      >
        <StyledNavCol sidenavBackgroundColor={props.sidenavBackgroundColor}>
          <Button
            onClick={props.handleToggle}
            bsPrefix="nav-button center nav-button-tour"
          >
            <motion.div animate={props.openMenu ? 'open' : 'closed'}>
              <MenuToggleComponent className="nav-menu-icon" />
            </motion.div>
          </Button>
          <hr className="m-0" />
          <NavButton
            onClick={() => history.push('/')}
            icon="fal fa-table"
            path="/"
            id="Example Section"
            className="node-nav-tour"
          />
          {/* LEAVE THIS LAST AS IT IS THE INFO PANEL */}
          <NavButton
            onClick={() => {
              if (
                props.openMenu !== true &&
                history.location.pathname !== '/pod-calculator'
              ) {
                props.handleToggle();
              }
              setIsTourOpen(true);
            }}
            icon="fal fa-info"
            id="Need-Help"
            className="help-nav-tour"
          />
        </StyledNavCol>
        <Col className="main-body">{props.children}</Col>
        <Tour
          steps={steps}
          isOpen={isTourOpen}
          onRequestClose={() => {
            setIsTourOpen(false);
            props.dispatch(saveUserWalkthroughAction(tourKey));
          }}
          maskSpace={1}
          rounded={5}
          startAt={0}
          accentColor="#23a5da"
        />
      </Container>
    </>
  );
}

NavContainer.propTypes = {
  dispatch: PropTypes.func.isRequired,
};

const mapStateToProps = createStructuredSelector({
  navContainer: makeSelectNavContainer(),
  reusableNotificationContainer: makeSelectReusableNotificationContainer(),
});

function mapDispatchToProps(dispatch) {
  return {
    dispatch,
  };
}

const withConnect = connect(
  mapStateToProps,
  mapDispatchToProps,
);

export default compose(
  injectReducer({
    key: 'navContainer',
    reducer,
  }),
  injectSaga({
    key: 'navContainer',
    saga,
  }),
  injectReducer({
    key: 'reusableNotificationContainer',
    reducer: reusableNotificationContainerReducer,
  }),
  injectSaga({
    key: 'reusableNotification',
    saga: reusableNotificationContainerSaga,
  }),
  withConnect,
  memo,
)(NavContainer);
